#include <stdexcept>
#include <regex>

#include "LayerStackBuilder.hh"

#include "loggers.hh"

LayerStackBuilder* LayerStackBuilder::_instance = NULL;

// static functions
LayerStackBuilder * LayerStackBuilder::get_instance()
{
  return _instance ? _instance : _instance = new LayerStackBuilder();
}

void LayerStackBuilder::register_layer_factory(const std::string & p_type, LayerFactory* p_layer_factory)
{
  LayerStackBuilder::get_instance()->_register_layer_factory(p_type, p_layer_factory);
}

// member functions
LayerStackBuilder::LayerStackBuilder()
{
}

void LayerStackBuilder::_register_layer_factory(const std::string & p_type, LayerFactory* p_layer_factory)
{
  _layer_factories[p_type] = p_layer_factory;
}

Layer* LayerStackBuilder::create_layer_stack(const char* p_layer_stack_description)
{
  loggers::get_instance().log(">>> LayerStackBuilder::create_layer_stack: %s", p_layer_stack_description);

  Layer * entry = NULL; // Initial layer (the first declared)
  Layer * up = NULL; // Upper layer
  // Parse the layer description
  try {
    std::regex rgx ("(\\w+)(\\((.*?)\\))?(\\/|$)");
    std::string str = p_layer_stack_description;
    std::sregex_iterator begin(str.cbegin(), str.cend(), rgx);
    std::sregex_iterator end = std::sregex_iterator();
    for (std::sregex_iterator it = begin; it != end; ++it) {
      std::smatch m = *it;
      loggers::get_instance().log("LayerStackBuilder::create_layer_stack: %d - %s - %s - %s - %s", m.size(), m[0].str().c_str(), m[1].str().c_str(), m[2].str().c_str(), m[3].str().c_str());
      LayerFactoryMap::iterator i = _layer_factories.find(m[1].str());
      if (i == _layer_factories.end()) {
        loggers::get_instance().error("LayerStackBuilder::create_layer_stack: %s: Unknown layer type", m[1].str().c_str());
      }
      loggers::get_instance().log("LayerStackBuilder::create_layer_stack: Create layer %s, %s", m[1].str().c_str(), m[3].str().c_str());
      Layer * l = i->second->create_layer(m[1].str(), m[3].str());
      if (NULL == l) {
        loggers::get_instance().error("LayerStackBuilder::create_layer_stack: %s: Layer creation error", m[1].str().c_str());
      }

      loggers::get_instance().log("LayerStackBuilder::create_layer_stack: Setup layers for %s", l->to_string().c_str());
      l->addUpperLayer(up);
      if (entry == NULL) { // Set the first declared layer
        entry = l;
      }
      up = l; // Build the linked list of layers
    } // End of 'for' statement
  }
  catch(const std::logic_error& e){
    if(up){ // FIXME To be reviewed
      up->deleteLayer();
      up = NULL;
    }
  }
  return entry;
}
